@@ -1,11 +1,10 @@
-use gpgpu::primitives::pixels::Rgba8Uint;
+use gpgpu::{primitives::pixels::Rgba8Uint, ImgOps};
 
 // This example simply mirrors an image using the image crate compatibility feature.
 fn main() {
     let fw = gpgpu::Framework::default();
-    let shader_mod =
-        gpgpu::utils::shader::from_wgsl_file(&fw, "examples/image-compatibility/mirror.wgsl")
-            .unwrap();
+    let shader =
+        gpgpu::Shader::from_wgsl_file(&fw, "examples/image-compatibility/shader.wgsl").unwrap();
 
     let dynamic_img = image::open("examples/image-compatibility/monke.jpg").unwrap(); // RGB8 image ...
     let rgba = dynamic_img.into_rgba8(); // ... converted to RGBA8
@@ -16,16 +15,14 @@ fn main() {
     let input_img = gpgpu::GpuConstImage::from_image_buffer(&fw, &rgba); // Input
     let output_img = gpgpu::GpuImage::<Rgba8Uint>::new(&fw, width, height); // Output
 
-    let binds = gpgpu::DescriptorSet::default()
+    let desc = gpgpu::DescriptorSet::default()
         .bind_const_image(&input_img)
         .bind_image(&output_img);
+    let program = gpgpu::Program::new(&shader, "main").add_descriptor_set(desc);
 
-    fw.create_kernel_builder(&shader_mod, "main")
-        .add_descriptor_set(binds)
-        .build()
-        .enqueue(width / 32, height / 32, 1); // Since the kernel workgroup size is (32,32,1) dims are divided
+    gpgpu::Kernel::new(&fw, program).enqueue(width / 32, height / 32, 1); // Since the kernel workgroup size is (32,32,1) dims are divided
 
-    let output = output_img.read_to_image_buffer().unwrap();
+    let output = output_img.read_to_image_buffer_blocking().unwrap();
     output
         .save("examples/image-compatibility/mirror-monke.png")
         .unwrap();
