@@ -1,6 +1,9 @@
+use std::{borrow::Cow, path::Path};
+
 use crate::{
-    primitives::PixelInfo, DescriptorSet, GpuBuffer, GpuBufferUsage, GpuConstImage, GpuImage,
-    GpuUniformBuffer, Kernel, KernelBuilder,
+    primitives::{BufOps, ImgOps, PixelInfo},
+    DescriptorSet, Framework, GpuBuffer, GpuBufferUsage, GpuConstImage, GpuImage, GpuUniformBuffer,
+    Kernel, Program, Shader,
 };
 
 impl<'res> DescriptorSet<'res> {
@@ -8,7 +11,6 @@ impl<'res> DescriptorSet<'res> {
     ///
     /// ### Example WGSL syntax:
     /// ```ignore
-    /// [[block]]
     /// struct UniformStruct {
     ///     a: vec3<u32>;
     ///     b: vec3<u32>;
@@ -47,7 +49,7 @@ impl<'res> DescriptorSet<'res> {
 
         let bind = wgpu::BindGroupEntry {
             binding: bind_id,
-            resource: uniform_buf.0.get_binding_resource(),
+            resource: uniform_buf.as_binding_resource(),
         };
 
         self.set_layout.push(bind_entry);
@@ -56,13 +58,10 @@ impl<'res> DescriptorSet<'res> {
         self
     }
 
-    /// Binds a [`GpuBuffer`] as a storage buffer in the shader.
-    ///
-    /// The `access` mode must be either [`AccessMode::ReadOnly`] or [`AccessMode::ReadWrite`].
+    /// Binds a [`GpuBuffer`] as a storage buffer in the shader with a specific `usage`.
     ///
     /// ### Example WGSL syntax:
     /// ```ignore
-    /// [[block]]
     /// struct StorageStruct {
     ///     data: [[stride(4)]] array<i32>;
     /// };
@@ -77,7 +76,7 @@ impl<'res> DescriptorSet<'res> {
     ///     int data[];
     /// };
     /// ```
-    pub fn bind_buffer<T>(mut self, storage_buf: &'res GpuBuffer<T>, access: GpuBufferUsage) -> Self
+    pub fn bind_buffer<T>(mut self, storage_buf: &'res GpuBuffer<T>, usage: GpuBufferUsage) -> Self
     where
         T: bytemuck::Pod,
     {
@@ -90,7 +89,7 @@ impl<'res> DescriptorSet<'res> {
                 has_dynamic_offset: false,
                 min_binding_size: None,
                 ty: wgpu::BufferBindingType::Storage {
-                    read_only: access == GpuBufferUsage::ReadOnly,
+                    read_only: usage == GpuBufferUsage::ReadOnly,
                 },
             },
             count: None,
@@ -98,7 +97,7 @@ impl<'res> DescriptorSet<'res> {
 
         let bind = wgpu::BindGroupEntry {
             binding: bind_id,
-            resource: storage_buf.0.get_binding_resource(),
+            resource: storage_buf.as_binding_resource(),
         };
 
         self.set_layout.push(bind_entry);
@@ -135,7 +134,7 @@ impl<'res> DescriptorSet<'res> {
 
         let bind = wgpu::BindGroupEntry {
             binding: bind_id,
-            resource: img.0.get_binding_resource(),
+            resource: img.as_binding_resource(),
         };
 
         self.set_layout.push(bind_entry);
@@ -175,7 +174,7 @@ impl<'res> DescriptorSet<'res> {
 
         let bind = wgpu::BindGroupEntry {
             binding: bind_id,
-            resource: img.0.get_binding_resource(),
+            resource: img.as_binding_resource(),
         };
 
         self.set_layout.push(bind_entry);
@@ -185,67 +184,114 @@ impl<'res> DescriptorSet<'res> {
     }
 }
 
-impl<'fw, 'res, 'sha> KernelBuilder<'fw, 'res, 'sha> {
-    /// Adds a [`DescriptorSet`] into the [`KernelBuilder`] internal layout.
+impl Shader {
+    /// Initialises a [`Shader`] from a SPIR-V file.
+    pub fn from_spirv_file(fw: &Framework, path: impl AsRef<Path>) -> std::io::Result<Self> {
+        let bytes = std::fs::read(&path)?;
+        let shader_name = path.as_ref().to_str();
+
+        Ok(Self::from_spirv_bytes(fw, &bytes, shader_name))
+    }
+
+    /// Initialises a [`Shader`] from SPIR-V bytes with an optional `name`.
+    pub fn from_spirv_bytes(fw: &Framework, bytes: &[u8], name: Option<&str>) -> Self {
+        let source = wgpu::util::make_spirv(bytes);
+
+        let shader = fw
+            .device
+            .create_shader_module(&wgpu::ShaderModuleDescriptor {
+                label: name,
+                source,
+            });
+
+        Self(shader)
+    }
+
+    /// Initialises a [`Shader`] from a `WGSL` file.
+    pub fn from_wgsl_file(fw: &Framework, path: impl AsRef<Path>) -> std::io::Result<Self> {
+        let source_string = std::fs::read_to_string(&path)?;
+        let shader_name = path.as_ref().to_str();
+
+        Ok(Self(fw.device.create_shader_module(
+            &wgpu::ShaderModuleDescriptor {
+                label: shader_name,
+                source: wgpu::ShaderSource::Wgsl(Cow::Owned(source_string)),
+            },
+        )))
+    }
+}
+
+impl<'sha, 'res> Program<'sha, 'res> {
+    /// Creates a new [`Program`] using a `shader` and an `entry_point`.
+    pub fn new(shader: &'sha Shader, entry_point: impl Into<String>) -> Self {
+        Self {
+            shader,
+            entry_point: entry_point.into(),
+            descriptors: Vec::new(),
+        }
+    }
+
+    /// Adds a [`DescriptorSet`] to this [`Program`] layout.
     pub fn add_descriptor_set(mut self, desc: DescriptorSet<'res>) -> Self {
-        let set_layout =
-            self.fw
+        self.descriptors.push(desc);
+        self
+    }
+}
+
+impl<'fw> Kernel<'fw> {
+    /// Creates a [`Kernel`] from a [`Program`].
+    pub fn new<'sha, 'res>(fw: &'fw Framework, program: Program<'sha, 'res>) -> Self {
+        let mut layouts = Vec::new();
+        let mut sets = Vec::new();
+
+        // Unwraping of descriptors from program
+        for desc in &program.descriptors {
+            let set_layout = fw
                 .device
                 .create_bind_group_layout(&wgpu::BindGroupLayoutDescriptor {
                     label: None,
                     entries: &desc.set_layout,
                 });
 
-        let set = self
-            .fw
-            .device
-            .create_bind_group(&wgpu::BindGroupDescriptor {
+            let set = fw.device.create_bind_group(&wgpu::BindGroupDescriptor {
                 label: None,
                 layout: &set_layout,
                 entries: &desc.binds,
             });
 
-        self.layouts.push(set_layout);
-        self.descriptors.push(desc);
-        self.sets.push(set);
+            layouts.push(set_layout);
+            sets.push(set);
+        }
 
-        self
-    }
+        // Compute pipeline bindings
+        let group_layouts = layouts.iter().collect::<Vec<_>>();
 
-    /// Builds a [`Kernel`] from the layout stored in [`KernelBuilder`].
-    pub fn build(self) -> Kernel<'fw> {
-        let sets = self.layouts.iter().collect::<Vec<_>>();
+        let pipeline_layout = fw
+            .device
+            .create_pipeline_layout(&wgpu::PipelineLayoutDescriptor {
+                label: None,
+                bind_group_layouts: &group_layouts,
+                push_constant_ranges: &[],
+            });
 
-        let pipeline_layout =
-            self.fw
-                .device
-                .create_pipeline_layout(&wgpu::PipelineLayoutDescriptor {
-                    label: None,
-                    bind_group_layouts: &sets,
-                    push_constant_ranges: &[],
-                });
-
-        let pipeline = self
-            .fw
+        let pipeline = fw
             .device
             .create_compute_pipeline(&wgpu::ComputePipelineDescriptor {
                 label: None,
-                module: self.shader,
-                entry_point: &self.entry_point,
+                module: &program.shader.0,
+                entry_point: &program.entry_point,
                 layout: Some(&pipeline_layout),
             });
 
-        Kernel {
-            fw: self.fw,
+        Self {
+            fw,
             pipeline,
-            sets: self.sets,
-            entry_point: self.entry_point,
+            sets,
+            entry_point: program.entry_point,
         }
     }
-}
 
-impl<'fw> Kernel<'fw> {
-    /// Enqueues the execution of this [`Kernel`] to the GPU.
+    /// Enqueues the execution of this [`Kernel`] onto the GPU.
     ///
     /// [`Kernel`] will dispatch `x`, `y` and `z` workgroups per dimension.
     pub fn enqueue(&self, x: u32, y: u32, z: u32) {
