@@ -1,7 +1,7 @@
-//! A simple GPU compute library based on [`wgpu`](https://github.com/gfx-rs/wgpu).
+//! An experimental async GPU compute library based on [`wgpu`](https://github.com/gfx-rs/wgpu).
 //! It is meant to be used alongside `wgpu` if desired.
 //!
-//! To start using `gpgpu`, just create a [`Framework`] instance
+//! To start using `gpgpu`, just create a [`Framework`](crate::Framework) instance
 //! and follow the [examples](https://github.com/UpsettingBoy/gpgpu-rs/tree/dev/examples) in the main repository.
 //!
 //! # Example
@@ -9,37 +9,35 @@
 //! result in another vector C.
 //! ## Rust program
 //! ```no_run
-//!  use gpgpu::*;
-//!
-//!  fn main() -> GpuResult<()> {
+//!     use gpgpu::*;
+//! fn main() -> Result<(), Box<dyn std::error::Error>> {
+//!     // Framework initialization
 //!     let fw = Framework::default();
-//!     
+//!
 //!     // Original CPU data
 //!     let cpu_data = (0..10000).into_iter().collect::<Vec<u32>>();
 //!
 //!     // GPU buffer creation
 //!     let buf_a = GpuBuffer::from_slice(&fw, &cpu_data);       // Input
 //!     let buf_b = GpuBuffer::from_slice(&fw, &cpu_data);       // Input
-//!     let buf_c = GpuBuffer::<u32>::new(&fw, cpu_data.len());  // Output
+//!     let buf_c = GpuBuffer::<u32>::with_capacity(&fw, cpu_data.len() as u64);  // Output
 //!
 //!     // Shader load from SPIR-V binary file
-//!     let shader_module = utils::shader::from_spirv_file(&fw, "<SPIR-V shader path>")?;
+//!     let shader = Shader::from_spirv_file(&fw, "<SPIR-V shader path>")?;
 //!     //  or from a WGSL source file
-//!     let shader_module = utils::shader::from_wgsl_file(&fw, "<WGSL shader path>")?;    
+//!     let shader = Shader::from_wgsl_file(&fw, "<WGSL shader path>")?;    
 //!
-//!     // Descriptor set creation
-//!     let desc_set = DescriptorSet::default()
+//!     // Descriptor set and program creation
+//!     let desc = DescriptorSet::default()
 //!         .bind_buffer(&buf_a, GpuBufferUsage::ReadOnly)
 //!         .bind_buffer(&buf_b, GpuBufferUsage::ReadOnly)
 //!         .bind_buffer(&buf_c, GpuBufferUsage::ReadWrite);
-//!     
-//!     // Kernel creation and enqueuing
-//!     fw.create_kernel_builder(&shader_module, "main")   // Entry point
-//!         .add_descriptor_set(desc_set)                      
-//!         .build()
-//!         .enqueue(cpu_data.len() as u32, 1, 1);         // Enqueuing, not very optimus 😅
+//!     let program = Program::new(&shader, "main").add_descriptor_set(desc); // Entry point
 //!
-//!     let output = buf_c.read()?;                        // Read back C from GPU
+//!     // Kernel creation and enqueuing
+//!     Kernel::new(&fw, program).enqueue(cpu_data.len() as u32, 1, 1); // Enqueuing, not very optimus 😅
+//!
+//!     let output = buf_c.read_vec_blocking()?;                        // Read back C from GPU
 //!     for (a, b) in cpu_data.into_iter().zip(output) {
 //!         assert_eq!(a.pow(2), b);
 //!     }
@@ -49,10 +47,9 @@
 //! ```
 //!
 //! ## Shader program
-//! The shader is writen in [WGSL](https://gpuweb.github.io/gpuweb/wgsl/)
+//! The shader is written in [WGSL](https://gpuweb.github.io/gpuweb/wgsl/)
 //! ```ignore
 //! // Vector type definition. Used for both input and output
-//! [[block]]
 //! struct Vector {
 //!     data: [[stride(4)]] array<u32>;
 //! };
@@ -69,25 +66,22 @@
 //! ```
 //!
 
-use primitives::{generic_buffer::GenericBuffer, generic_image::GenericImage};
+use std::{marker::PhantomData, sync::Arc};
+
+#[cfg(feature = "integrate-ndarray")]
+pub use features::integrate_ndarray::GpuArray;
+pub use primitives::{BufOps, ImgOps};
 
 pub mod features;
 pub mod framework;
 pub mod kernel;
 pub mod primitives;
-pub mod utils;
-
-/// Lazy error handling :)
-pub type GpuResult<T> = Result<T, Box<dyn std::error::Error>>;
 
 /// Entry point of `gpgpu`. A [`Framework`] must be created
 /// first as all GPU primitives needs it to be created.
-#[allow(dead_code)]
 pub struct Framework {
-    instance: wgpu::Instance,
-    pub device: wgpu::Device,
-    pub queue: wgpu::Queue,
-    limits: wgpu::Limits,
+    device: Arc<wgpu::Device>,
+    queue: wgpu::Queue,
 }
 
 #[derive(PartialEq, Eq)]
@@ -113,7 +107,12 @@ pub enum GpuBufferUsage {
 ///
 /// More information about its shader representation is
 /// under the [`DescriptorSet::bind_buffer`](crate::DescriptorSet::bind_buffer) documentation.
-pub struct GpuBuffer<'fw, T: bytemuck::Pod>(GenericBuffer<'fw, T>);
+pub struct GpuBuffer<'fw, T> {
+    fw: &'fw Framework,
+    buf: wgpu::Buffer,
+    size: u64,
+    marker: PhantomData<T>,
+}
 
 /// Uniform vector of contiguous homogeneous elements on GPU memory.
 /// Recommended for small, read-only buffers.
@@ -123,7 +122,12 @@ pub struct GpuBuffer<'fw, T: bytemuck::Pod>(GenericBuffer<'fw, T>);
 ///
 /// More information about its shader representation is
 /// under the [`DescriptorSet::bind_uniform_buffer`](crate::DescriptorSet::bind_uniform_buffer) documentation.
-pub struct GpuUniformBuffer<'fw, T: bytemuck::Pod>(GenericBuffer<'fw, T>);
+pub struct GpuUniformBuffer<'fw, T> {
+    fw: &'fw Framework,
+    buf: wgpu::Buffer,
+    size: u64,
+    marker: PhantomData<T>,
+}
 
 /// 2D-image of homogeneous pixels.
 ///
@@ -131,7 +135,13 @@ pub struct GpuUniformBuffer<'fw, T: bytemuck::Pod>(GenericBuffer<'fw, T>);
 ///
 /// More information about its shader representation is
 /// under the [`DescriptorSet::bind_image`](crate::DescriptorSet::bind_image) documentation.
-pub struct GpuImage<'fw, P>(GenericImage<'fw, P>);
+pub struct GpuImage<'fw, P> {
+    fw: &'fw Framework,
+    texture: wgpu::Texture,
+    size: wgpu::Extent3d,
+    full_view: wgpu::TextureView,
+    pixel: PhantomData<P>,
+}
 
 /// 2D-image of homogeneous pixels.
 ///
@@ -139,29 +149,35 @@ pub struct GpuImage<'fw, P>(GenericImage<'fw, P>);
 ///
 /// More information about its shader representation is
 /// under the [`DescriptorSet::bind_const_image`](crate::DescriptorSet::bind_const_image) documentation.
-pub struct GpuConstImage<'fw, P>(GenericImage<'fw, P>);
+pub struct GpuConstImage<'fw, P> {
+    fw: &'fw Framework,
+    texture: wgpu::Texture,
+    size: wgpu::Extent3d,
+    full_view: wgpu::TextureView,
+    pixel: PhantomData<P>,
+}
+
+/// Represents a shader.
+///
+/// It's just a wrapper over [`wgpu::ShaderModule`].
+pub struct Shader(wgpu::ShaderModule);
+
+/// Represents an entry point with its bindings on a [`Shader`].
+pub struct Program<'sha, 'res> {
+    shader: &'sha Shader,
+    entry_point: String,
+    descriptors: Vec<DescriptorSet<'res>>,
+}
 
 /// Contains a binding group of resources.
-#[derive(Default)]
+#[derive(Default, Clone)]
 pub struct DescriptorSet<'res> {
     set_layout: Vec<wgpu::BindGroupLayoutEntry>,
     binds: Vec<wgpu::BindGroupEntry<'res>>,
 }
 
-/// Creates a [`Kernel`] instance with the bindings
-/// used during the configuration of this structure.
-pub struct KernelBuilder<'fw, 'res, 'sha> {
-    fw: &'fw Framework,
-    layouts: Vec<wgpu::BindGroupLayout>,
-    descriptors: Vec<DescriptorSet<'res>>,
-    sets: Vec<wgpu::BindGroup>,
-    shader: &'sha wgpu::ShaderModule,
-    entry_point: String,
-}
-
 /// Used to enqueue the execution of a shader with the bidings provided.
 ///
-/// Can only be created from [`KernelBuilder`].
 /// Equivalent to OpenCL's Kernel.
 pub struct Kernel<'fw> {
     fw: &'fw Framework,
